package cn.iocoder.yudao.module.system.service.positionmap;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.api.remote.dto.RemoteRobotDTO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.dto.PositionMapDTO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.vo.PositionMapSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapDO;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * 仓库点位地图 Service 接口
 *
 * @author 芋道源码
 */
public interface PositionMapService extends IService<PositionMapDO> {

    /**
     * 创建仓库点位地图
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createPositionMap(@Valid PositionMapSaveReqVO createReqVO);

    /**
     * 更新仓库点位地图
     *
     * @param updateReqVO 更新信息
     */
    void updatePositionMap(@Valid PositionMapSaveReqVO updateReqVO);

    /**
     * 删除仓库点位地图
     *
     * @param id 编号
     */
    void deletePositionMap(Long id);

    /**
     * 获得仓库点位地图
     *
     * @param id 编号
     * @return 仓库点位地图
     */
    PositionMapDO getPositionMap(Long id);

    /**
     * 获得仓库点位地图分页
     *
     * @param pageReqVO 分页查询
     * @return 仓库点位地图分页
     */
    PageResult<PositionMapDO> getPositionMapPage(PositionMapPageReqVO pageReqVO);

    /**
     * AGV文件上传
     *
     * @param files
     */
    void uploadFiles(MultipartFile[] files);

    /**
     * 文件下载 - 指定地图文件
     *
     * @param floor
     * @param area
     * @param response
     */
    void downloadFile(Integer floor, String area, HttpServletResponse response) throws IOException;

    /**
     * AGV文件下载 - 所有地图文件
     *
     * @param response
     * @throws IOException
     */
    void downloadFile(HttpServletResponse response) throws IOException;

    /**
     * 前端下载图片信息
     *
     * @param floor
     * @param area
     */
    String downloadPngBase64(Integer floor, String area);

    /**
     * 一键急停or一键恢复
     *
     * @param id
     */
    void emergencyStopOrRecovery(Long id,Integer type,Boolean isRemote);

    /**
     * 获取所有地图
     * @return
     */
    List<PositionMapDO> getAllMap();

    /**
     * 获取地图及显示地图是否暂停
     * @return
     */
    Map<Integer, List<PositionMapDTO>> getPositionMapList();

    /**
     * 远遥获取地图和车辆信息
     * @param mapId
     * @return
     */
    List<RemoteRobotDTO> getRemoteAreaRobotByMapId(Long mapId);
}
